# coding: utf-8

"""
    VNS3 Controller API

    Cohesive networks VNS3 API providing complete control of your network's addresses, routes, rules and edge  # noqa: E501

    The version of the OpenAPI document: 4.8
    Contact: solutions@cohesive.net
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

import datetime
import pytest

from tests.rest_mock import RestClientMock

from cohesivenet import VNS3Client, models, Configuration
from cohesivenet.rest import ApiException


class TestMonitoringAlertingApi(object):
    """MonitoringAlertingApi unit test stubs"""

    def test_delete_alert(self):
        """Test case for delete_alert

        """
        pass

    def test_get_alert(self):
        """Test case for get_alert

        """
        pass

    def test_get_alerts(self):
        """Test case for get_alerts

        """
        pass

    def test_post_define_new_alert(self):
        """Test case for post_define_new_alert

        """
        pass

    def test_post_test_alert(self):
        """Test case for post_test_alert

        """
        pass

    def test_post_toggle_enabled_alert(self):
        """Test case for post_toggle_enabled_alert

        """
        pass

    def test_put_update_alert(self):
        """Test case for put_update_alert

        """
        pass

    def test_delete_webhook(self):
        """Test case for delete_webhook

        """
        pass

    def test_get_webhook(self):
        """Test case for get_webhook

        """
        pass

    def test_get_webhooks(self, rest_mock: RestClientMock):
        """Test case for get_webhooks

        """
        method, uri = "get", "/api/webhooks"
        rest_mock.stub_request(
            method,
            uri,
            {
                "response": [
                    {
                        "body": "{\n"
                        '    "attachments": [{\n'
                        '      "author_name": "Tunnel %{status}: '
                        '%{tunnel_description}",\n'
                        '      "author_link": "%{tunnel_url}",\n'
                        '      "fields": [\n'
                        '        {"title": "VNS3 Controller", "value": '
                        '"%{topo_name}" , "short": true},\n'
                        '        {"title": "Endpoint", "value": "%{ipsec_name} '
                        '(%{ipsec_ip})", "short": true},\n'
                        '        {"title": "Local Subnet", "value": '
                        '"%{local_subnet}", "short": true},\n'
                        '        {"title": "Remote Subnet", "value": '
                        '"%{remote_subnet}", "short": true}\n'
                        "      ]\n"
                        "    }]\n"
                        "  }",
                        "created_at": "2019-11-12T16:03:28.554000",
                        "custom_properties": [],
                        "events": ["tunnel_up", "tunnel_down"],
                        "headers": [],
                        "id": 2,
                        "name": "Test Slack",
                        "parameters": [],
                        "updated_at": "2019-11-12T16:03:28.554000",
                        "url": "",
                        "validate_cert": True,
                    }
                ]
            },
        )

        api_client = VNS3Client(
            configuration=Configuration(
                host="0.0.0.0:8000",
                username="api",
                password="password",
                verify_ssl=False,
            )
        )

        resp = api_client.monitoring.get_webhooks()
        assert type(resp) is models.WebhooksListResponse
        assert resp.response[0].id == 2

    def test_post_create_webhook(self):
        """Test case for post_create_webhook

        """
        pass

    def test_put_update_webhook(self):
        """Test case for put_update_webhook

        """
        pass



# coding: utf-8

"""
    VNS3 Controller API

    Cohesive networks VNS3 API providing complete control of your network's addresses, routes, rules and edge  # noqa: E501

    The version of the OpenAPI document: 4.8
    Contact: solutions@cohesive.net
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class CreateAlertRequest(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "description": "str",
        "url": "str",
        "webhook_id": "int",
        "events": "list[str]",
        "custom_properties": "dict(str, str)",
        "enabled": "bool",
    }

    attribute_map = {
        "description": "description",
        "url": "url",
        "webhook_id": "webhook_id",
        "events": "events",
        "custom_properties": "custom_properties",
        "enabled": "enabled",
    }

    def __init__(
        self,
        description=None,
        url=None,
        webhook_id=None,
        events=None,
        custom_properties=None,
        enabled=True,
    ):  # noqa: E501
        """CreateAlertRequest - a model defined in OpenAPI"""  # noqa: E501

        self._description = None
        self._url = None
        self._webhook_id = None
        self._events = None
        self._custom_properties = None
        self._enabled = None
        self.discriminator = None

        self.description = description
        self.url = url
        self.webhook_id = webhook_id
        if events is not None:
            self.events = events
        if custom_properties is not None:
            self.custom_properties = custom_properties
        if enabled is not None:
            self.enabled = enabled

    @property
    def description(self):
        """Gets the description of this CreateAlertRequest.  # noqa: E501


        :return: The description of this CreateAlertRequest.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this CreateAlertRequest.


        :param description: The description of this CreateAlertRequest.  # noqa: E501
        :type: str
        """
        if description is None:
            raise ValueError(
                "Invalid value for `description`, must not be `None`"
            )  # noqa: E501

        self._description = description

    @property
    def url(self):
        """Gets the url of this CreateAlertRequest.  # noqa: E501


        :return: The url of this CreateAlertRequest.  # noqa: E501
        :rtype: str
        """
        return self._url

    @url.setter
    def url(self, url):
        """Sets the url of this CreateAlertRequest.


        :param url: The url of this CreateAlertRequest.  # noqa: E501
        :type: str
        """
        if url is None:
            raise ValueError(
                "Invalid value for `url`, must not be `None`"
            )  # noqa: E501

        self._url = url

    @property
    def webhook_id(self):
        """Gets the webhook_id of this CreateAlertRequest.  # noqa: E501


        :return: The webhook_id of this CreateAlertRequest.  # noqa: E501
        :rtype: int
        """
        return self._webhook_id

    @webhook_id.setter
    def webhook_id(self, webhook_id):
        """Sets the webhook_id of this CreateAlertRequest.


        :param webhook_id: The webhook_id of this CreateAlertRequest.  # noqa: E501
        :type: int
        """
        if webhook_id is None:
            raise ValueError(
                "Invalid value for `webhook_id`, must not be `None`"
            )  # noqa: E501

        self._webhook_id = webhook_id

    @property
    def events(self):
        """Gets the events of this CreateAlertRequest.  # noqa: E501


        :return: The events of this CreateAlertRequest.  # noqa: E501
        :rtype: list[str]
        """
        return self._events

    @events.setter
    def events(self, events):
        """Sets the events of this CreateAlertRequest.


        :param events: The events of this CreateAlertRequest.  # noqa: E501
        :type: list[str]
        """

        self._events = events

    @property
    def custom_properties(self):
        """Gets the custom_properties of this CreateAlertRequest.  # noqa: E501


        :return: The custom_properties of this CreateAlertRequest.  # noqa: E501
        :rtype: dict(str, str)
        """
        return self._custom_properties

    @custom_properties.setter
    def custom_properties(self, custom_properties):
        """Sets the custom_properties of this CreateAlertRequest.


        :param custom_properties: The custom_properties of this CreateAlertRequest.  # noqa: E501
        :type: dict(str, str)
        """

        self._custom_properties = custom_properties

    @property
    def enabled(self):
        """Gets the enabled of this CreateAlertRequest.  # noqa: E501


        :return: The enabled of this CreateAlertRequest.  # noqa: E501
        :rtype: bool
        """
        return self._enabled

    @enabled.setter
    def enabled(self, enabled):
        """Sets the enabled of this CreateAlertRequest.


        :param enabled: The enabled of this CreateAlertRequest.  # noqa: E501
        :type: bool
        """

        self._enabled = enabled

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(
                    map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value)
                )
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict")
                        else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, CreateAlertRequest):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



from cohesivenet import util, Logger


def create_firewall_policy(client, firewall_rules, state={}):
    """Create group of firewall rules

    Arguments:
        client {VNS3Client}
        firewall_rules {List[CreateFirewallRuleRequest]} - [{
            'position': int,
            'rule': str
        }, ...]

    Keyword Arguments:
        state {dict} - State to format rules with. (can call client.controller_state)

    Returns:
        Tuple[List[str], List[str]] - success, errors
    """
    successes = []
    errors = []
    Logger.debug(
        "Creating firewall policy.",
        host=client.host_uri,
        rule_count=len(firewall_rules),
    )
    for i, rule_args in enumerate(firewall_rules):
        rule = rule_args["rule"]
        if util.is_formattable_string(rule):
            try:
                rule = rule.format(**state)
                rule = rule.format(**state)
                rule_args.update(rule=rule)
            except KeyError as e:
                errors.append("Rule %d missing state args %s" % (i, ",".join(e.args)))
                continue

        client.firewall.post_create_firewall_rule(rule_args)
        successes.append(
            'Rule "%s" inserted at position %d' % (rule, rule_args["position"])
        )
    return successes, errors



from __future__ import absolute_import

# flake8: noqa

# import apis into api package
from cohesivenet.api.vns3.bgp_api import BGPApi
from cohesivenet.api.vns3.configuration_api import ConfigurationApi
from cohesivenet.api.vns3.firewall_api import FirewallApi
from cohesivenet.api.vns3.high_availability_api import HighAvailabilityApi
from cohesivenet.api.vns3.ipsec_api import IPsecApi
from cohesivenet.api.vns3.interfaces_api import InterfacesApi
from cohesivenet.api.vns3.licensing_api import LicensingApi
from cohesivenet.api.vns3.monitoring_alerting_api import MonitoringAlertingApi
from cohesivenet.api.vns3.network_edge_plugins_api import NetworkEdgePluginsApi
from cohesivenet.api.vns3.overlay_network_api import OverlayNetworkApi
from cohesivenet.api.vns3.peering_api import PeeringApi
from cohesivenet.api.vns3.routing_api import RoutingApi
from cohesivenet.api.vns3.snapshots_api import SnapshotsApi
from cohesivenet.api.vns3.system_administration_api import SystemAdministrationApi



# coding: utf-8

"""
    VNS3 Controller API

    Cohesive networks VNS3 API providing complete control of your network's addresses, routes, rules and edge  # noqa: E501

    The version of the OpenAPI document: 4.8
    Contact: solutions@cohesive.net
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class StartContainerRequest(object):
    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """

    openapi_types = {
        "uuid": "str",
        "image_uuid": "str",
        "ipaddress": "str",
        "name": "str",
        "description": "str",
        "command": "str",
        "environment": "str",
    }

    attribute_map = {
        "uuid": "uuid",
        "image_uuid": "image_uuid",
        "ipaddress": "ipaddress",
        "name": "name",
        "description": "description",
        "command": "command",
        "environment": "environment",
    }

    def __init__(
        self,
        uuid=None,
        image_uuid=None,
        name=None,
        ipaddress=None,
        description=None,
        command=None,
        environment=None,
    ):  # noqa: E501
        """StartContainerRequest - a model defined in OpenAPI"""  # noqa: E501

        self._uuid = None
        self._image_uuid = None
        self._name = None
        self._ipaddress = None
        self._description = None
        self._command = None
        self._environment = None
        self.discriminator = None

        if uuid is not None:
            self.uuid = uuid
        if image_uuid is not None:
            self.image_uuid = image_uuid
        if name is not None:
            self.name = name
        if ipaddress is not None:
            self.ipaddress = ipaddress
        if description is not None:
            self.description = description
        if command is not None:
            self.command = command
        if environment is not None:
            self.environment = environment

    @property
    def uuid(self):
        """Gets the uuid of this StartContainerRequest.  # noqa: E501


        :return: The uuid of this StartContainerRequest.  # noqa: E501
        :rtype: str
        """
        return self._uuid

    @uuid.setter
    def uuid(self, uuid):
        """Sets the uuid of this StartContainerRequest.


        :param uuid: The uuid of this StartContainerRequest.  # noqa: E501
        :type: str
        """

        self._uuid = uuid

    @property
    def image_uuid(self):
        """Gets the image_uuid of this StartContainerRequest.  # noqa: E501


        :return: The image_uuid of this StartContainerRequest.  # noqa: E501
        :rtype: str
        """
        return self._image_uuid

    @image_uuid.setter
    def image_uuid(self, image_uuid):
        """Sets the image_uuid of this StartContainerRequest.


        :param image_uuid: The image_uuid of this StartContainerRequest.  # noqa: E501
        :type: str
        """

        self._image_uuid = image_uuid

    @property
    def name(self):
        """Gets the name of this StartContainerRequest.  # noqa: E501


        :return: The name of this StartContainerRequest.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this StartContainerRequest.


        :param name: The name of this StartContainerRequest.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def ipaddress(self):
        """Gets the ipaddress of this StartContainerRequest.  # noqa: E501


        :return: The ipaddress of this StartContainerRequest.  # noqa: E501
        :rtype: str
        """
        return self._ipaddress

    @ipaddress.setter
    def ipaddress(self, ipaddress):
        """Sets the ipaddress of this StartContainerRequest.


        :param ipaddress: The ipaddress of this StartContainerRequest.  # noqa: E501
        :type: str
        """

        self._ipaddress = ipaddress

    @property
    def description(self):
        """Gets the description of this StartContainerRequest.  # noqa: E501


        :return: The description of this StartContainerRequest.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this StartContainerRequest.


        :param description: The description of this StartContainerRequest.  # noqa: E501
        :type: str
        """

        self._description = description

    @property
    def command(self):
        """Gets the command of this StartContainerRequest.  # noqa: E501


        :return: The command of this StartContainerRequest.  # noqa: E501
        :rtype: str
        """
        return self._command

    @command.setter
    def command(self, command):
        """Sets the command of this StartContainerRequest.


        :param command: The command of this StartContainerRequest.  # noqa: E501
        :type: str
        """

        self._command = command

    @property
    def environment(self):
        """Gets the environment of this StartContainerRequest.  # noqa: E501


        :return: The environment of this StartContainerRequest.  # noqa: E501
        :rtype: str
        """
        return self._environment

    @environment.setter
    def environment(self, environment):
        """Sets the environment of this StartContainerRequest.


        :param environment: The environment of this StartContainerRequest.  # noqa: E501
        :type: str
        """

        self._environment = environment

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(
                    map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value)
                )
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict")
                        else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, StartContainerRequest):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    VNS3 Controller API

    Cohesive networks VNS3 API providing complete control of your network's addresses, routes, rules and edge  # noqa: E501

    The version of the OpenAPI document: 4.8
    Contact: solutions@cohesive.net
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class UpdateLicenseParametersRequest(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "subnet": "str",
        "managers": "str",
        "asns": "str",
        "clients": "str",
        "my_manager_vip": "str",
        "default": "bool",
    }

    attribute_map = {
        "subnet": "subnet",
        "managers": "managers",
        "asns": "asns",
        "clients": "clients",
        "my_manager_vip": "my_manager_vip",
        "default": "default",
    }

    def __init__(
        self,
        subnet=None,
        managers=None,
        asns=None,
        clients=None,
        my_manager_vip=None,
        default=None,
    ):  # noqa: E501
        """UpdateLicenseParametersRequest - a model defined in OpenAPI"""  # noqa: E501

        self._subnet = None
        self._managers = None
        self._asns = None
        self._clients = None
        self._my_manager_vip = None
        self._default = None
        self.discriminator = None

        if subnet is not None:
            self.subnet = subnet
        if managers is not None:
            self.managers = managers
        if asns is not None:
            self.asns = asns
        if clients is not None:
            self.clients = clients
        if my_manager_vip is not None:
            self.my_manager_vip = my_manager_vip
        if default is not None:
            self.default = default

    @property
    def subnet(self):
        """Gets the subnet of this UpdateLicenseParametersRequest.  # noqa: E501

        Specifies the CIDR of the virtual network created for use with the VNS3 Manager  # noqa: E501

        :return: The subnet of this UpdateLicenseParametersRequest.  # noqa: E501
        :rtype: str
        """
        return self._subnet

    @subnet.setter
    def subnet(self, subnet):
        """Sets the subnet of this UpdateLicenseParametersRequest.

        Specifies the CIDR of the virtual network created for use with the VNS3 Manager  # noqa: E501

        :param subnet: The subnet of this UpdateLicenseParametersRequest.  # noqa: E501
        :type: str
        """

        self._subnet = subnet

    @property
    def managers(self):
        """Gets the managers of this UpdateLicenseParametersRequest.  # noqa: E501

        Whitespace delimited address string in the subnet to use for the VNS3 Controllers' virtual interfaces.  # noqa: E501

        :return: The managers of this UpdateLicenseParametersRequest.  # noqa: E501
        :rtype: str
        """
        return self._managers

    @managers.setter
    def managers(self, managers):
        """Sets the managers of this UpdateLicenseParametersRequest.

        Whitespace delimited address string in the subnet to use for the VNS3 Controllers' virtual interfaces.  # noqa: E501

        :param managers: The managers of this UpdateLicenseParametersRequest.  # noqa: E501
        :type: str
        """

        self._managers = managers

    @property
    def asns(self):
        """Gets the asns of this UpdateLicenseParametersRequest.  # noqa: E501

        Whitespace delimited string of ASNs (autonomous system numbers) corresponding to the order of the controllers  # noqa: E501

        :return: The asns of this UpdateLicenseParametersRequest.  # noqa: E501
        :rtype: str
        """
        return self._asns

    @asns.setter
    def asns(self, asns):
        """Sets the asns of this UpdateLicenseParametersRequest.

        Whitespace delimited string of ASNs (autonomous system numbers) corresponding to the order of the controllers  # noqa: E501

        :param asns: The asns of this UpdateLicenseParametersRequest.  # noqa: E501
        :type: str
        """

        self._asns = asns

    @property
    def clients(self):
        """Gets the clients of this UpdateLicenseParametersRequest.  # noqa: E501

        Comma delimited, or hyphenated sequence of addresses for use as client addresses in the virtual network.  # noqa: E501

        :return: The clients of this UpdateLicenseParametersRequest.  # noqa: E501
        :rtype: str
        """
        return self._clients

    @clients.setter
    def clients(self, clients):
        """Sets the clients of this UpdateLicenseParametersRequest.

        Comma delimited, or hyphenated sequence of addresses for use as client addresses in the virtual network.  # noqa: E501

        :param clients: The clients of this UpdateLicenseParametersRequest.  # noqa: E501
        :type: str
        """

        self._clients = clients

    @property
    def my_manager_vip(self):
        """Gets the my_manager_vip of this UpdateLicenseParametersRequest.  # noqa: E501

        IPAddress that must be allocated from the subnet, and be the same for all controllers  # noqa: E501

        :return: The my_manager_vip of this UpdateLicenseParametersRequest.  # noqa: E501
        :rtype: str
        """
        return self._my_manager_vip

    @my_manager_vip.setter
    def my_manager_vip(self, my_manager_vip):
        """Sets the my_manager_vip of this UpdateLicenseParametersRequest.

        IPAddress that must be allocated from the subnet, and be the same for all controllers  # noqa: E501

        :param my_manager_vip: The my_manager_vip of this UpdateLicenseParametersRequest.  # noqa: E501
        :type: str
        """

        self._my_manager_vip = my_manager_vip

    @property
    def default(self):
        """Gets the default of this UpdateLicenseParametersRequest.  # noqa: E501

        Specifices whether to use defualt topology addressing as specified by the license  # noqa: E501

        :return: The default of this UpdateLicenseParametersRequest.  # noqa: E501
        :rtype: bool
        """
        return self._default

    @default.setter
    def default(self, default):
        """Sets the default of this UpdateLicenseParametersRequest.

        Specifices whether to use defualt topology addressing as specified by the license  # noqa: E501

        :param default: The default of this UpdateLicenseParametersRequest.  # noqa: E501
        :type: bool
        """

        self._default = default

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(
                    map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value)
                )
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict")
                        else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, UpdateLicenseParametersRequest):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    VNS3 Controller API

    Cohesive networks VNS3 API providing complete control of your network's addresses, routes, rules and edge  # noqa: E501

    The version of the OpenAPI document: 4.8
    Contact: solutions@cohesive.net
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class CreateContainerImageRequest(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "name": "str",
        "url": "str",
        "buildurl": "str",
        "localbuild": "str",
        "localimage": "str",
        "description": "str",
    }

    attribute_map = {
        "name": "name",
        "url": "url",
        "buildurl": "buildurl",
        "localbuild": "localbuild",
        "localimage": "localimage",
        "description": "description",
    }

    def __init__(
        self,
        name=None,
        url=None,
        buildurl=None,
        localbuild=None,
        localimage=None,
        description=None,
    ):  # noqa: E501
        """CreateContainerImageRequest - a model defined in OpenAPI"""  # noqa: E501

        self._name = None
        self._url = None
        self._buildurl = None
        self._localbuild = None
        self._localimage = None
        self._description = None
        self.discriminator = None

        if name is not None:
            self.name = name
        if url is not None:
            self.url = url
        if buildurl is not None:
            self.buildurl = buildurl
        if localbuild is not None:
            self.localbuild = localbuild
        if localimage is not None:
            self.localimage = localimage
        if description is not None:
            self.description = description

    @property
    def name(self):
        """Gets the name of this CreateContainerImageRequest.  # noqa: E501

        Name of the image  # noqa: E501

        :return: The name of this CreateContainerImageRequest.  # noqa: E501
        :rtype: str
        """
        return self._expires

    @name.setter
    def name(self, name):
        """Sets the name of this CreateContainerImageRequest.

        Name of the image  # noqa: E501

        :param name: The name of this CreateContainerImageRequest.  # noqa: E501
        :type: str
        """
        if name is None:
            raise ValueError(
                "Invalid value for `name`, must not be `None`"
            )  # noqa: E501

        self._name = name

    @property
    def url(self):
        """Gets the url of this CreateContainerImageRequest.  # noqa: E501

        URL of the image file to be imported # noqa: E501

        :return: The url of this CreateContainerImageRequest.  # noqa: E501
        :rtype: str
        """
        return self._url

    @url.setter
    def url(self, url):
        """Sets the URL of this CreateContainerImageRequest.

        URL of the image file to be imported  # noqa: E501

        :param url: The url of this CreateContainerImageRequest.  # noqa: E501
        :type: str
        """
        self._url = url

    @property
    def buildurl(self):
        """Gets the buildurl of this CreateContainerImageRequest.  # noqa: E501

        URL of a dockerfile that will be used to build the image # noqa: E501

        :return: The buildurl of this CreateContainerImageRequest.  # noqa: E501
        :rtype: str
        """
        return self._url

    @buildurl.setter
    def buildurl(self, buildurl):
        """Sets the buildurl of this CreateContainerImageRequest.

        URL of a dockerfile that will be used to build the image  # noqa: E501

        :param buildurl: The buildurl of this CreateContainerImageRequest.  # noqa: E501
        :type: str
        """
        self._buildurl = buildurl

    @property
    def localbuild(self):
        """Gets the localbuild of this CreateContainerImageRequest.  # noqa: E501

        Local build file to create new image # noqa: E501

        :return: The localbuild of this CreateContainerImageRequest.  # noqa: E501
        :rtype: str
        """
        return self._localbuild

    @localbuild.setter
    def localbuild(self, localbuild):
        """Sets the localbuild of this CreateContainerImageRequest.

        Local build file to create new image  # noqa: E501

        :param localbuild: The localbuild of this CreateContainerImageRequest.  # noqa: E501
        :type: str
        """
        self._localbuild = localbuild

    @property
    def localimage(self):
        """Gets the localimage of this CreateContainerImageRequest.  # noqa: E501

        Local image to tag # noqa: E501

        :return: The localimage of this CreateContainerImageRequest.  # noqa: E501
        :rtype: str
        """
        return self._localimage

    @localimage.setter
    def localimage(self, localimage):
        """Sets the localimage of this CreateContainerImageRequest.

        Local image to tag  # noqa: E501

        :param localimage: The localimage of this CreateContainerImageRequest.  # noqa: E501
        :type: str
        """
        self._localimage = localimage

    @property
    def description(self):
        """Gets the description of this CreateContainerImageRequest.  # noqa: E501

        # noqa: E501

        :return: The description of this CreateContainerImageRequest.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this CreateContainerImageRequest.

         # noqa: E501

        :param description: The description of this CreateContainerImageRequest.  # noqa: E501
        :type: str
        """
        self._description = description

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(
                    map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value)
                )
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict")
                        else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, CreateContainerImageRequest):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    VNS3 Controller API

    Cohesive networks VNS3 API providing complete control of your network's addresses, routes, rules and edge  # noqa: E501

    The version of the OpenAPI document: 4.8
    Contact: solutions@cohesive.net
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class PeersDetail(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "id": "int",
        "peered": "bool",
        "managers": "dict(str, VNS3ControllerPeer)",
        "controllers": "dict(str, object)",
    }

    attribute_map = {
        "id": "id",
        "peered": "peered",
        "managers": "managers",
        "controllers": "controllers",
    }

    def __init__(
        self, id=None, peered=None, managers=None, controllers=None
    ):  # noqa: E501
        """PeersDetail - a model defined in OpenAPI"""  # noqa: E501

        self._id = None
        self._peered = None
        self._managers = None
        self._controllers = None
        self.discriminator = None

        if id is not None:
            self.id = id
        if peered is not None:
            self.peered = peered
        if managers is not None:
            self.managers = managers
        if controllers is not None:
            self.controllers = controllers

    @property
    def id(self):
        """Gets the id of this PeersDetail.  # noqa: E501


        :return: The id of this PeersDetail.  # noqa: E501
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this PeersDetail.


        :param id: The id of this PeersDetail.  # noqa: E501
        :type: int
        """

        self._id = id

    @property
    def peered(self):
        """Gets the peered of this PeersDetail.  # noqa: E501


        :return: The peered of this PeersDetail.  # noqa: E501
        :rtype: bool
        """
        return self._peered

    @peered.setter
    def peered(self, peered):
        """Sets the peered of this PeersDetail.


        :param peered: The peered of this PeersDetail.  # noqa: E501
        :type: bool
        """

        self._peered = peered

    @property
    def managers(self):
        """Gets the managers of this PeersDetail.  # noqa: E501


        :return: The managers of this PeersDetail.  # noqa: E501
        :rtype: dict(str, VNS3ControllerPeer)
        """
        return self._managers

    @managers.setter
    def managers(self, managers):
        """Sets the managers of this PeersDetail.


        :param managers: The managers of this PeersDetail.  # noqa: E501
        :type: dict(str, VNS3ControllerPeer)
        """

        self._managers = managers

    @property
    def controllers(self):
        """Gets the controllers of this PeersDetail.  # noqa: E501


        :return: The controllers of this PeersDetail.  # noqa: E501
        :rtype: dict(str, object)
        """
        return self._controllers

    @controllers.setter
    def controllers(self, controllers):
        """Sets the controllers of this PeersDetail.


        :param controllers: The controllers of this PeersDetail.  # noqa: E501
        :type: dict(str, object)
        """

        self._controllers = controllers

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(
                    map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value)
                )
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict")
                        else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, PeersDetail):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    VNS3 Controller API

    Cohesive networks VNS3 API providing complete control of your network's addresses, routes, rules and edge  # noqa: E501

    The version of the OpenAPI document: 4.8
    Contact: solutions@cohesive.net
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class RuntimeStatus(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "connected_clients": "dict(str, OverlayClient)",
        "connected_subnets": "list[list[str]]",
        "ipsec": "dict(str, IpsecTunnel)",
    }

    attribute_map = {
        "connected_clients": "connected_clients",
        "connected_subnets": "connected_subnets",
        "ipsec": "ipsec",
    }

    def __init__(
        self, connected_clients=None, connected_subnets=None, ipsec=None
    ):  # noqa: E501
        """RuntimeStatus - a model defined in OpenAPI"""  # noqa: E501

        self._connected_clients = None
        self._connected_subnets = None
        self._ipsec = None
        self.discriminator = None

        if connected_clients is not None:
            self.connected_clients = connected_clients
        if connected_subnets is not None:
            self.connected_subnets = connected_subnets
        if ipsec is not None:
            self.ipsec = ipsec

    @property
    def connected_clients(self):
        """Gets the connected_clients of this RuntimeStatus.  # noqa: E501

        clients keyed by ip address  # noqa: E501

        :return: The connected_clients of this RuntimeStatus.  # noqa: E501
        :rtype: dict(str, OverlayClient)
        """
        return self._connected_clients

    @connected_clients.setter
    def connected_clients(self, connected_clients):
        """Sets the connected_clients of this RuntimeStatus.

        clients keyed by ip address  # noqa: E501

        :param connected_clients: The connected_clients of this RuntimeStatus.  # noqa: E501
        :type: dict(str, OverlayClient)
        """

        self._connected_clients = connected_clients

    @property
    def connected_subnets(self):
        """Gets the connected_subnets of this RuntimeStatus.  # noqa: E501

        Array of arrays with each element of length 2 representing [network, subnet mask]  # noqa: E501

        :return: The connected_subnets of this RuntimeStatus.  # noqa: E501
        :rtype: list[list[str]]
        """
        return self._connected_subnets

    @connected_subnets.setter
    def connected_subnets(self, connected_subnets):
        """Sets the connected_subnets of this RuntimeStatus.

        Array of arrays with each element of length 2 representing [network, subnet mask]  # noqa: E501

        :param connected_subnets: The connected_subnets of this RuntimeStatus.  # noqa: E501
        :type: list[list[str]]
        """

        self._connected_subnets = connected_subnets

    @property
    def ipsec(self):
        """Gets the ipsec of this RuntimeStatus.  # noqa: E501

        IPSEC tunnels keyed by tunnel ID  # noqa: E501

        :return: The ipsec of this RuntimeStatus.  # noqa: E501
        :rtype: dict(str, IpsecTunnel)
        """
        return self._ipsec

    @ipsec.setter
    def ipsec(self, ipsec):
        """Sets the ipsec of this RuntimeStatus.

        IPSEC tunnels keyed by tunnel ID  # noqa: E501

        :param ipsec: The ipsec of this RuntimeStatus.  # noqa: E501
        :type: dict(str, IpsecTunnel)
        """

        self._ipsec = ipsec

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(
                    map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value)
                )
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict")
                        else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, RuntimeStatus):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other





def create_tunnel_endpoint(
    client,
    tunnel_name,
    tunnel_secret,
    target_ip,
    target_network_cidr,
    vti_block,
    target_network_name=None,
    tunnel_parameters={},
    route_parameters={},
):
    """create_tunnel_endpoint Create IPsec endpoint and route to target network

    Arguments:
        client {VNS3Client} -- VNS3 Api client
        tunnel_name {str} -- Name to be used for tunnel
        tunnel_secret {str} -- Pre-Shared Secret for use on both sides of tunnel
        target_ip {str} -- IP address of target endpoint
        target_network_cidr {str or ipaddress.IPv4Network} -- CIDR of target network for routing
        vti_block {str} -- CIDR to use for network interface

    Keyword Arguments:
        target_network_name {str} -- [description] (default: {None})
        tunnel_parameters {dict} -- parameters to use for IPsec endpoint (default: {{}})
        route_parameters {dict} -- parameters to use for route (default: {{}})

    Returns:
        [Tuple[IpsecRemoteEndpoint, Dict]]
    """
    ipsec_endpoint = client.ipsec.post_create_ipsec_endpoint(
        dict(
            {
                "name": tunnel_name,
                "ipaddress": target_ip,
                "secret": tunnel_secret,
                "pfs": True,
                "ike_version": 2,
                "nat_t_enabled": "False",  # # older versions of VNS3 required string parameters.
                "extra_config": "local-peer-id=%s" % client.configuration.host_uri,
                "vpn_type": "vti",
                "route_based_int_address": str(vti_block),
                "route_based_local": "0.0.0.0/0",
                "route_based_remote": "0.0.0.0/0",
            },
            **tunnel_parameters
        )
    )

    routes = client.routing.post_create_route(
        dict(
            {
                "cidr": target_network_cidr,
                "description": "Route to %s via tunnel" % target_network_name
                or target_network_cidr,
                "tunnel": list(ipsec_endpoint.response.tunnels.keys())[0],
                "advertise": "False",
                "gateway": "",
                "metric": 0,
            },
            **route_parameters
        )
    )

    return {"endpoint": ipsec_endpoint, "routes": routes}



# coding: utf-8

"""
    VNS3 Controller API

    Cohesive networks VNS3 API providing complete control of your network's addresses, routes, rules and edge  # noqa: E501

    The version of the OpenAPI document: 4.8
    Contact: solutions@cohesive.net
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class CreateAPITokenRequest(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {"expires": "int", "token_name": "str", "refreshes": "bool"}

    attribute_map = {
        "expires": "expires",
        "token_name": "token_name",
        "refreshes": "refreshes",
    }

    def __init__(self, expires=3600, token_name=None, refreshes=None):  # noqa: E501
        """CreateAPITokenRequest - a model defined in OpenAPI"""  # noqa: E501

        self._expires = None
        self._token_name = None
        self._refreshes = None
        self.discriminator = None

        if expires is not None:
            self.expires = expires
        if token_name is not None:
            self.token_name = token_name
        if refreshes is not None:
            self.refreshes = refreshes

    @property
    def expires(self):
        """Gets the expires of this CreateAPITokenRequest.  # noqa: E501

        Number of seconds before expiration  # noqa: E501

        :return: The expires of this CreateAPITokenRequest.  # noqa: E501
        :rtype: int
        """
        return self._expires

    @expires.setter
    def expires(self, expires):
        """Sets the expires of this CreateAPITokenRequest.

        Number of seconds before expiration  # noqa: E501

        :param expires: The expires of this CreateAPITokenRequest.  # noqa: E501
        :type: int
        """
        if expires is not None and expires > 31536000:  # noqa: E501
            raise ValueError(
                "Invalid value for `expires`, must be a value less than or equal to `31536000`"
            )  # noqa: E501
        if expires is not None and expires < 60:  # noqa: E501
            raise ValueError(
                "Invalid value for `expires`, must be a value greater than or equal to `60`"
            )  # noqa: E501

        self._expires = expires

    @property
    def token_name(self):
        """Gets the token_name of this CreateAPITokenRequest.  # noqa: E501

        Optional description of token  # noqa: E501

        :return: The token_name of this CreateAPITokenRequest.  # noqa: E501
        :rtype: str
        """
        return self._token_name

    @token_name.setter
    def token_name(self, token_name):
        """Sets the token_name of this CreateAPITokenRequest.

        Optional description of token  # noqa: E501

        :param token_name: The token_name of this CreateAPITokenRequest.  # noqa: E501
        :type: str
        """

        self._token_name = token_name

    @property
    def refreshes(self):
        """Gets the refreshes of this CreateAPITokenRequest.  # noqa: E501

        Token lifetime refreshes when used  # noqa: E501

        :return: The refreshes of this CreateAPITokenRequest.  # noqa: E501
        :rtype: bool
        """
        return self._refreshes

    @refreshes.setter
    def refreshes(self, refreshes):
        """Sets the refreshes of this CreateAPITokenRequest.

        Token lifetime refreshes when used  # noqa: E501

        :param refreshes: The refreshes of this CreateAPITokenRequest.  # noqa: E501
        :type: bool
        """

        self._refreshes = refreshes

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(
                    map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value)
                )
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict")
                        else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, CreateAPITokenRequest):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8
# To install the library, run the following
#
# python setup.py install
#
# prerequisite: setuptools
# http://pypi.python.org/pypi/setuptools

"""
    Cohesive Networks SDK

    Cohesive Networks SDK is a thin wrapper around our product APIs providing complete control of your network's addresses, routes, rules and edge  # noqa: E501

    The version of the OpenAPI document: 4.8
    Contact: solutions@cohesive.net
    Generated by: https://openapi-generator.tech
"""

import os
import sys
from setuptools import setup, find_packages  # noqa: H301
from setuptools.command.test import test as TestCommand

NAME = "cohesivenet"
DEPENDENDIES = [
    "urllib3 >= 1.15",
    "certifi >=  14.05.14",
    "python-dateutil >= 2.5.3",
    "paramiko >= 2.4.3",
]

DIR = os.path.abspath(os.path.dirname(__file__))


class PyTest(TestCommand):
    user_options = [("pytest-args=", "a", "Arguments to pass to pytest")]

    def initialize_options(self):
        TestCommand.initialize_options(self)
        self.pytest_args = ""

    def run_tests(self):
        import shlex

        # import here, cause outside the eggs aren't loaded
        import pytest

        errno = pytest.main(shlex.split(self.pytest_args))
        sys.exit(errno)


os.chdir(DIR)

version_contents = {}
with open(os.path.join(DIR, "cohesivenet", "version.py"), encoding="utf-8") as f:
    exec(f.read(), version_contents)


def load_long_description():
    with open(os.path.join(DIR, "README.md"), "r") as fh:
        return fh.read()


setup(
    name=NAME,
    version=version_contents["VERSION"],
    description="Cohesive Networks SDK",
    author="Cohesive Networks, Inc.",
    long_description=load_long_description(),
    long_description_content_type="text/markdown",
    author_email="solutions@cohesive.net",
    python_requires=">=3.5.0",
    url="https://github.com/cohesive/python-cohesivenet-sdk",
    keywords=[
        "Cohesive Networks SDK",
        "Cohesive Networks",
        "Secops",
        "SDN",
        "Software Defined Networking",
        "Networkops",
        "networking",
        "Openapi",
    ],
    install_requires=DEPENDENDIES,
    tests_require=[
        "pytest >= 4.6.2",
        "pytest-mock >= 1.10.4",
        "pytest-xdist >= 1.28.0",
        "pytest-cov >= 2.7.1",
    ],
    packages=find_packages(exclude=["tests", "docs", "examples"]),
    include_package_data=True,
    package_data={NAME: ["version"]},
    cmdclass={"test": PyTest},
    license="MIT",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "Topic :: System :: Networking",
        "Topic :: System :: Networking :: Firewalls",
        "Topic :: System :: Networking :: Monitoring",
        "Topic :: Security",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ],
)



# coding: utf-8

"""
    VNS3 Controller API

    Cohesive networks VNS3 API providing complete control of your network's addresses, routes, rules and edge  # noqa: E501

    The version of the OpenAPI document: 4.8
    Contact: solutions@cohesive.net
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class ContainerImage(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "id": "str",
        "image_name": "str",
        "status": "str",
        "status_msg": "str",
        "import_id": "str",
        "created": "datetime",
        "description": "str",
        "tag_name": "str",
        "comment": "str",
        "container_config": "dict(str, object)",
    }

    attribute_map = {
        "id": "id",
        "image_name": "image_name",
        "status": "status",
        "status_msg": "status_msg",
        "import_id": "import_id",
        "created": "created",
        "description": "description",
        "tag_name": "tag_name",
        "comment": "comment",
        "container_config": "container_config",
    }

    def __init__(
        self,
        id=None,
        image_name=None,
        status=None,
        status_msg=None,
        import_id=None,
        created=None,
        description=None,
        tag_name=None,
        comment=None,
        container_config=None,
    ):  # noqa: E501
        """ContainerImage - a model defined in OpenAPI"""  # noqa: E501

        self._id = None
        self._image_name = None
        self._status = None
        self._status_msg = None
        self._import_id = None
        self._created = None
        self._description = None
        self._tag_name = None
        self._comment = None
        self._container_config = None
        self.discriminator = None

        self.id = id
        if image_name is not None:
            self.image_name = image_name
        if status is not None:
            self.status = status
        if status_msg is not None:
            self.status_msg = status_msg
        if import_id is not None:
            self.import_id = import_id
        if created is not None:
            self.created = created
        if description is not None:
            self.description = description
        if tag_name is not None:
            self.tag_name = tag_name
        if comment is not None:
            self.comment = comment
        if container_config is not None:
            self.container_config = container_config

    @property
    def id(self):
        """Gets the id of this ContainerImage.  # noqa: E501


        :return: The id of this ContainerImage.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this ContainerImage.


        :param id: The id of this ContainerImage.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def image_name(self):
        """Gets the image_name of this ContainerImage.  # noqa: E501


        :return: The image_name of this ContainerImage.  # noqa: E501
        :rtype: str
        """
        return self._image_name

    @image_name.setter
    def image_name(self, image_name):
        """Sets the image_name of this ContainerImage.


        :param image_name: The image_name of this ContainerImage.  # noqa: E501
        :type: str
        """

        self._image_name = image_name

    @property
    def status(self):
        """Gets the status of this ContainerImage.  # noqa: E501


        :return: The status of this ContainerImage.  # noqa: E501
        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, status):
        """Sets the status of this ContainerImage.


        :param status: The status of this ContainerImage.  # noqa: E501
        :type: str
        """

        self._status = status

    @property
    def status_msg(self):
        """Gets the status_msg of this ContainerImage.  # noqa: E501


        :return: The status_msg of this ContainerImage.  # noqa: E501
        :rtype: str
        """
        return self._status_msg

    @status_msg.setter
    def status_msg(self, status_msg):
        """Sets the status_msg of this ContainerImage.


        :param status_msg: The status_msg of this ContainerImage.  # noqa: E501
        :type: str
        """

        self._status_msg = status_msg

    @property
    def import_id(self):
        """Gets the import_id of this ContainerImage.  # noqa: E501


        :return: The import_id of this ContainerImage.  # noqa: E501
        :rtype: str
        """
        return self._import_id

    @import_id.setter
    def import_id(self, import_id):
        """Sets the import_id of this ContainerImage.


        :param import_id: The import_id of this ContainerImage.  # noqa: E501
        :type: str
        """

        self._import_id = import_id

    @property
    def created(self):
        """Gets the created of this ContainerImage.  # noqa: E501


        :return: The created of this ContainerImage.  # noqa: E501
        :rtype: datetime
        """
        return self._created

    @created.setter
    def created(self, created):
        """Sets the created of this ContainerImage.


        :param created: The created of this ContainerImage.  # noqa: E501
        :type: datetime
        """

        self._created = created

    @property
    def description(self):
        """Gets the description of this ContainerImage.  # noqa: E501


        :return: The description of this ContainerImage.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this ContainerImage.


        :param description: The description of this ContainerImage.  # noqa: E501
        :type: str
        """

        self._description = description

    @property
    def tag_name(self):
        """Gets the tag_name of this ContainerImage.  # noqa: E501


        :return: The tag_name of this ContainerImage.  # noqa: E501
        :rtype: str
        """
        return self._tag_name

    @tag_name.setter
    def tag_name(self, tag_name):
        """Sets the tag_name of this ContainerImage.


        :param tag_name: The tag_name of this ContainerImage.  # noqa: E501
        :type: str
        """

        self._tag_name = tag_name

    @property
    def comment(self):
        """Gets the comment of this ContainerImage.  # noqa: E501


        :return: The comment of this ContainerImage.  # noqa: E501
        :rtype: str
        """
        return self._comment

    @comment.setter
    def comment(self, comment):
        """Sets the comment of this ContainerImage.


        :param comment: The comment of this ContainerImage.  # noqa: E501
        :type: str
        """

        self._comment = comment

    @property
    def container_config(self):
        """Gets the container_config of this ContainerImage.  # noqa: E501


        :return: The container_config of this ContainerImage.  # noqa: E501
        :rtype: dict(str, object)
        """
        return self._container_config

    @container_config.setter
    def container_config(self, container_config):
        """Sets the container_config of this ContainerImage.


        :param container_config: The container_config of this ContainerImage.  # noqa: E501
        :type: dict(str, object)
        """

        self._container_config = container_config

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(
                    map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value)
                )
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict")
                        else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ContainerImage):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



from typing import Tuple, List
from collections import namedtuple

ClientExceptionResult = namedtuple("ClientExceptionResult", "client exception")
OperationResult = namedtuple("OperationResult", "client result")
BulkOperationResult = Tuple[List[OperationResult], List[ClientExceptionResult]]



# coding: utf-8

"""
    VNS3 Controller API

    Cohesive networks VNS3 API providing complete control of your network's addresses, routes, rules and edge  # noqa: E501

    The version of the OpenAPI document: 4.8
    Contact: solutions@cohesive.net
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class IpsecTunnel(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "id": "int",
        "local_subnet": "str",
        "remote_subnet": "str",
        "endpoint_id": "int",
        "endpoint_name": "str",
        "active": "bool",
        "description": "str",
        "connected": "bool",
        "ping_interface": "str",
        "ping_interval": "int",
        "ping_ipaddress": "str",
        "tunnel_params": "IpsecTunnelParams",
    }

    attribute_map = {
        "id": "id",
        "local_subnet": "local_subnet",
        "remote_subnet": "remote_subnet",
        "endpoint_id": "endpoint_id",
        "endpoint_name": "endpoint_name",
        "active": "active",
        "description": "description",
        "connected": "connected",
        "ping_interface": "ping_interface",
        "ping_interval": "ping_interval",
        "ping_ipaddress": "ping_ipaddress",
        "tunnel_params": "tunnel_params",
    }

    def __init__(
        self,
        id=None,
        local_subnet=None,
        remote_subnet=None,
        endpoint_id=None,
        endpoint_name=None,
        active=None,
        description=None,
        connected=None,
        ping_interface=None,
        ping_interval=None,
        ping_ipaddress=None,
        tunnel_params=None,
    ):  # noqa: E501
        """IpsecTunnel - a model defined in OpenAPI"""  # noqa: E501

        self._id = None
        self._local_subnet = None
        self._remote_subnet = None
        self._endpoint_id = None
        self._endpoint_name = None
        self._active = None
        self._description = None
        self._connected = None
        self._ping_interface = None
        self._ping_interval = None
        self._ping_ipaddress = None
        self._tunnel_params = None
        self.discriminator = None

        if id is not None:
            self.id = id
        if local_subnet is not None:
            self.local_subnet = local_subnet
        if remote_subnet is not None:
            self.remote_subnet = remote_subnet
        if endpoint_id is not None:
            self.endpoint_id = endpoint_id
        if endpoint_name is not None:
            self.endpoint_name = endpoint_name
        if active is not None:
            self.active = active
        if description is not None:
            self.description = description
        if connected is not None:
            self.connected = connected
        if ping_interface is not None:
            self.ping_interface = ping_interface
        if ping_interval is not None:
            self.ping_interval = ping_interval
        if ping_ipaddress is not None:
            self.ping_ipaddress = ping_ipaddress
        if tunnel_params is not None:
            self.tunnel_params = tunnel_params

    @property
    def id(self):
        """Gets the id of this IpsecTunnel.  # noqa: E501


        :return: The id of this IpsecTunnel.  # noqa: E501
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this IpsecTunnel.


        :param id: The id of this IpsecTunnel.  # noqa: E501
        :type: int
        """

        self._id = id

    @property
    def local_subnet(self):
        """Gets the local_subnet of this IpsecTunnel.  # noqa: E501


        :return: The local_subnet of this IpsecTunnel.  # noqa: E501
        :rtype: str
        """
        return self._local_subnet

    @local_subnet.setter
    def local_subnet(self, local_subnet):
        """Sets the local_subnet of this IpsecTunnel.


        :param local_subnet: The local_subnet of this IpsecTunnel.  # noqa: E501
        :type: str
        """

        self._local_subnet = local_subnet

    @property
    def remote_subnet(self):
        """Gets the remote_subnet of this IpsecTunnel.  # noqa: E501


        :return: The remote_subnet of this IpsecTunnel.  # noqa: E501
        :rtype: str
        """
        return self._remote_subnet

    @remote_subnet.setter
    def remote_subnet(self, remote_subnet):
        """Sets the remote_subnet of this IpsecTunnel.


        :param remote_subnet: The remote_subnet of this IpsecTunnel.  # noqa: E501
        :type: str
        """

        self._remote_subnet = remote_subnet

    @property
    def endpoint_id(self):
        """Gets the endpoint_id of this IpsecTunnel.  # noqa: E501


        :return: The endpoint_id of this IpsecTunnel.  # noqa: E501
        :rtype: int
        """
        return self._endpoint_id

    @endpoint_id.setter
    def endpoint_id(self, endpoint_id):
        """Sets the endpoint_id of this IpsecTunnel.


        :param endpoint_id: The endpoint_id of this IpsecTunnel.  # noqa: E501
        :type: int
        """

        self._endpoint_id = endpoint_id

    @property
    def endpoint_name(self):
        """Gets the endpoint_name of this IpsecTunnel.  # noqa: E501


        :return: The endpoint_name of this IpsecTunnel.  # noqa: E501
        :rtype: str
        """
        return self._endpoint_name

    @endpoint_name.setter
    def endpoint_name(self, endpoint_name):
        """Sets the endpoint_name of this IpsecTunnel.


        :param endpoint_name: The endpoint_name of this IpsecTunnel.  # noqa: E501
        :type: str
        """

        self._endpoint_name = endpoint_name

    @property
    def active(self):
        """Gets the active of this IpsecTunnel.  # noqa: E501


        :return: The active of this IpsecTunnel.  # noqa: E501
        :rtype: bool
        """
        return self._active

    @active.setter
    def active(self, active):
        """Sets the active of this IpsecTunnel.


        :param active: The active of this IpsecTunnel.  # noqa: E501
        :type: bool
        """

        self._active = active

    @property
    def description(self):
        """Gets the description of this IpsecTunnel.  # noqa: E501


        :return: The description of this IpsecTunnel.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this IpsecTunnel.


        :param description: The description of this IpsecTunnel.  # noqa: E501
        :type: str
        """

        self._description = description

    @property
    def connected(self):
        """Gets the connected of this IpsecTunnel.  # noqa: E501


        :return: The connected of this IpsecTunnel.  # noqa: E501
        :rtype: bool
        """
        return self._connected

    @connected.setter
    def connected(self, connected):
        """Sets the connected of this IpsecTunnel.


        :param connected: The connected of this IpsecTunnel.  # noqa: E501
        :type: bool
        """

        self._connected = connected

    @property
    def ping_interface(self):
        """Gets the ping_interface of this IpsecTunnel.  # noqa: E501


        :return: The ping_interface of this IpsecTunnel.  # noqa: E501
        :rtype: str
        """
        return self._ping_interface

    @ping_interface.setter
    def ping_interface(self, ping_interface):
        """Sets the ping_interface of this IpsecTunnel.


        :param ping_interface: The ping_interface of this IpsecTunnel.  # noqa: E501
        :type: str
        """
        allowed_values = ["eth0", "tun0"]  # noqa: E501
        if ping_interface not in allowed_values:
            raise ValueError(
                "Invalid value for `ping_interface` ({0}), must be one of {1}".format(  # noqa: E501
                    ping_interface, allowed_values
                )
            )

        self._ping_interface = ping_interface

    @property
    def ping_interval(self):
        """Gets the ping_interval of this IpsecTunnel.  # noqa: E501

        Interval for ping in seconds  # noqa: E501

        :return: The ping_interval of this IpsecTunnel.  # noqa: E501
        :rtype: int
        """
        return self._ping_interval

    @ping_interval.setter
    def ping_interval(self, ping_interval):
        """Sets the ping_interval of this IpsecTunnel.

        Interval for ping in seconds  # noqa: E501

        :param ping_interval: The ping_interval of this IpsecTunnel.  # noqa: E501
        :type: int
        """

        self._ping_interval = ping_interval

    @property
    def ping_ipaddress(self):
        """Gets the ping_ipaddress of this IpsecTunnel.  # noqa: E501


        :return: The ping_ipaddress of this IpsecTunnel.  # noqa: E501
        :rtype: str
        """
        return self._ping_ipaddress

    @ping_ipaddress.setter
    def ping_ipaddress(self, ping_ipaddress):
        """Sets the ping_ipaddress of this IpsecTunnel.


        :param ping_ipaddress: The ping_ipaddress of this IpsecTunnel.  # noqa: E501
        :type: str
        """

        self._ping_ipaddress = ping_ipaddress

    @property
    def tunnel_params(self):
        """Gets the tunnel_params of this IpsecTunnel.  # noqa: E501


        :return: The tunnel_params of this IpsecTunnel.  # noqa: E501
        :rtype: IpsecTunnelParams
        """
        return self._tunnel_params

    @tunnel_params.setter
    def tunnel_params(self, tunnel_params):
        """Sets the tunnel_params of this IpsecTunnel.


        :param tunnel_params: The tunnel_params of this IpsecTunnel.  # noqa: E501
        :type: IpsecTunnelParams
        """

        self._tunnel_params = tunnel_params

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(
                    map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value)
                )
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict")
                        else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, IpsecTunnel):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    VNS3 Controller API

    Cohesive networks VNS3 API providing complete control of your network's addresses, routes, rules and edge  # noqa: E501

    The version of the OpenAPI document: 4.8
    Contact: solutions@cohesive.net
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

import re  # noqa: F401

# python 2 and python 3 compatibility library
import six

from cohesivenet.exceptions import ApiTypeError, ApiValueError


class FirewallApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            from cohesivenet.vns3_client import VNS3Client

            api_client = VNS3Client()
        self.api_client = api_client

    def delete_firewall_fw_set(self, body, **kwargs):  # noqa: E501
        """delete_firewall_fw_set  # noqa: E501

        Delete Firewall FWSet by name or rules  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_firewall_fw_set(body, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param object body: (required)
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: object
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        return self.delete_firewall_fw_set_with_http_info(body, **kwargs)  # noqa: E501

    def delete_firewall_fw_set_with_http_info(self, body, **kwargs):  # noqa: E501
        """delete_firewall_fw_set  # noqa: E501

        Delete Firewall FWSet by name or rules  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_firewall_fw_set_with_http_info(body, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param object body: (required)
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: tuple(object, status_code(int), headers(HTTPHeaderDict))
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = ["body"]  # noqa: E501
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        for key, val in six.iteritems(local_var_params["kwargs"]):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method delete_firewall_fw_set" % key
                )
            local_var_params[key] = val
        del local_var_params["kwargs"]
        # verify the required parameter 'body' is set
        if "body" not in local_var_params or local_var_params["body"] is None:
            raise ApiValueError(
                "Missing the required parameter `body` when calling `delete_firewall_fw_set`"
            )  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if "body" in local_var_params:
            body_params = local_var_params["body"]
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # HTTP header `Content-Type`
        header_params[
            "Content-Type"
        ] = self.api_client.select_header_content_type(  # noqa: E501
            ["application/json"]
        )  # noqa: E501

        # Authentication setting
        auth_settings = ["basicAuth"]  # noqa: E501

        return self.api_client.call_api(
            "/firewall/fwsets",
            "DELETE",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type="object",  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get("async_req"),
            _return_http_data_only=local_var_params.get(
                "_return_http_data_only"
            ),  # noqa: E501
            _preload_content=local_var_params.get("_preload_content", True),
            _request_timeout=local_var_params.get("_request_timeout"),
            collection_formats=collection_formats,
        )

    def delete_firewall_rule_by_position(self, position, **kwargs):  # noqa: E501
        """delete_firewall_rule_by_position  # noqa: E501

        Delete Firewall Rule  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_firewall_rule_by_position(position, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param int position: index position for firewall rule, 0 is first (required)
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: object
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        return self.delete_firewall_rule_by_position_with_http_info(
            position, **kwargs
        )  # noqa: E501

    def delete_firewall_rule_by_position_with_http_info(
        self, position, **kwargs
    ):  # noqa: E501
        """delete_firewall_rule_by_position  # noqa: E501

        Delete Firewall Rule  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_firewall_rule_by_position_with_http_info(position, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param int position: index position for firewall rule, 0 is first (required)
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: tuple(object, status_code(int), headers(HTTPHeaderDict))
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = ["position"]  # noqa: E501
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        for key, val in six.iteritems(local_var_params["kwargs"]):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method delete_firewall_rule_by_position" % key
                )
            local_var_params[key] = val
        del local_var_params["kwargs"]
        # verify the required parameter 'position' is set
        if "position" not in local_var_params or local_var_params["position"] is None:
            raise ApiValueError(
                "Missing the required parameter `position` when calling `delete_firewall_rule_by_position`"
            )  # noqa: E501

        collection_formats = {}

        path_params = {}
        if "position" in local_var_params:
            path_params["position"] = local_var_params["position"]  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # Authentication setting
        auth_settings = ["basicAuth"]  # noqa: E501

        return self.api_client.call_api(
            "/firewall/rules/{position}",
            "DELETE",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type="object",  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get("async_req"),
            _return_http_data_only=local_var_params.get(
                "_return_http_data_only"
            ),  # noqa: E501
            _preload_content=local_var_params.get("_preload_content", True),
            _request_timeout=local_var_params.get("_request_timeout"),
            collection_formats=collection_formats,
        )

    def delete_firewall_rule_by_rule(
        self, delete_firewall_rule_request, **kwargs
    ):  # noqa: E501
        """delete_firewall_rule_by_rule  # noqa: E501

        Delete firewall rule  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_firewall_rule_by_rule(delete_firewall_rule_request, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param DeleteFirewallRuleRequest delete_firewall_rule_request: (required)
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: object
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        return self.delete_firewall_rule_by_rule_with_http_info(
            delete_firewall_rule_request, **kwargs
        )  # noqa: E501

    def delete_firewall_rule_by_rule_with_http_info(
        self, delete_firewall_rule_request, **kwargs
    ):  # noqa: E501
        """delete_firewall_rule_by_rule  # noqa: E501

        Delete firewall rule  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_firewall_rule_by_rule_with_http_info(delete_firewall_rule_request, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param DeleteFirewallRuleRequest delete_firewall_rule_request: (required)
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: tuple(object, status_code(int), headers(HTTPHeaderDict))
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = ["delete_firewall_rule_request"]  # noqa: E501
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        for key, val in six.iteritems(local_var_params["kwargs"]):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method delete_firewall_rule_by_rule" % key
                )
            local_var_params[key] = val
        del local_var_params["kwargs"]
        # verify the required parameter 'delete_firewall_rule_request' is set
        if (
            "delete_firewall_rule_request" not in local_var_params
            or local_var_params["delete_firewall_rule_request"] is None
        ):
            raise ApiValueError(
                "Missing the required parameter `delete_firewall_rule_request` when calling `delete_firewall_rule_by_rule`"
            )  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if "delete_firewall_rule_request" in local_var_params:
            body_params = local_var_params["delete_firewall_rule_request"]
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # HTTP header `Content-Type`
        header_params[
            "Content-Type"
        ] = self.api_client.select_header_content_type(  # noqa: E501
            ["application/json"]
        )  # noqa: E501

        # Authentication setting
        auth_settings = ["basicAuth"]  # noqa: E501

        return self.api_client.call_api(
            "/firewall/rules",
            "DELETE",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type="object",  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get("async_req"),
            _return_http_data_only=local_var_params.get(
                "_return_http_data_only"
            ),  # noqa: E501
            _preload_content=local_var_params.get("_preload_content", True),
            _request_timeout=local_var_params.get("_request_timeout"),
            collection_formats=collection_formats,
        )

    def delete_firewall_subgroup(self, firewall_fw_set, **kwargs):  # noqa: E501
        """delete_firewall_subgroup  # noqa: E501

        Delete Firewall subgroup by name or rules  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_firewall_subgroup(firewall_fw_set, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param FirewallFWSet firewall_fw_set: (required)
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: object
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        return self.delete_firewall_subgroup_with_http_info(
            firewall_fw_set, **kwargs
        )  # noqa: E501

    def delete_firewall_subgroup_with_http_info(
        self, firewall_fw_set, **kwargs
    ):  # noqa: E501
        """delete_firewall_subgroup  # noqa: E501

        Delete Firewall subgroup by name or rules  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_firewall_subgroup_with_http_info(firewall_fw_set, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param FirewallFWSet firewall_fw_set: (required)
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: tuple(object, status_code(int), headers(HTTPHeaderDict))
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = ["firewall_fw_set"]  # noqa: E501
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        for key, val in six.iteritems(local_var_params["kwargs"]):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method delete_firewall_subgroup" % key
                )
            local_var_params[key] = val
        del local_var_params["kwargs"]
        # verify the required parameter 'firewall_fw_set' is set
        if (
            "firewall_fw_set" not in local_var_params
            or local_var_params["firewall_fw_set"] is None
        ):
            raise ApiValueError(
                "Missing the required parameter `firewall_fw_set` when calling `delete_firewall_subgroup`"
            )  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if "firewall_fw_set" in local_var_params:
            body_params = local_var_params["firewall_fw_set"]
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # HTTP header `Content-Type`
        header_params[
            "Content-Type"
        ] = self.api_client.select_header_content_type(  # noqa: E501
            ["application/json"]
        )  # noqa: E501

        # Authentication setting
        auth_settings = ["basicAuth"]  # noqa: E501

        return self.api_client.call_api(
            "/firewall/rules/subgroup",
            "DELETE",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type="object",  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get("async_req"),
            _return_http_data_only=local_var_params.get(
                "_return_http_data_only"
            ),  # noqa: E501
            _preload_content=local_var_params.get("_preload_content", True),
            _request_timeout=local_var_params.get("_request_timeout"),
            collection_formats=collection_formats,
        )

    def get_firewall_fw_sets(self, **kwargs):  # noqa: E501
        """get_firewall_fw_sets  # noqa: E501

        Get a list of current firewall rule sets. These are IPsets that allow for faster matching of rules against IPs.  See http://ipset.netfilter.org for more details.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_firewall_fw_sets(async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: FirewallFWSetListResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        return self.get_firewall_fw_sets_with_http_info(**kwargs)  # noqa: E501

    def get_firewall_fw_sets_with_http_info(self, **kwargs):  # noqa: E501
        """get_firewall_fw_sets  # noqa: E501

        Get a list of current firewall rule sets. These are IPsets that allow for faster matching of rules against IPs.  See http://ipset.netfilter.org for more details.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_firewall_fw_sets_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: tuple(FirewallFWSetListResponse, status_code(int), headers(HTTPHeaderDict))
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = []  # noqa: E501
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        for key, val in six.iteritems(local_var_params["kwargs"]):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_firewall_fw_sets" % key
                )
            local_var_params[key] = val
        del local_var_params["kwargs"]

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # Authentication setting
        auth_settings = ["basicAuth"]  # noqa: E501

        return self.api_client.call_api(
            "/firewall/fwsets",
            "GET",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type="FirewallFWSetListResponse",  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get("async_req"),
            _return_http_data_only=local_var_params.get(
                "_return_http_data_only"
            ),  # noqa: E501
            _preload_content=local_var_params.get("_preload_content", True),
            _request_timeout=local_var_params.get("_request_timeout"),
            collection_formats=collection_formats,
        )

    def get_firewall_rule_subgroups(self, **kwargs):  # noqa: E501
        """get_firewall_rule_subgroups  # noqa: E501

        Get a list of current firewall rules at subgroup (chained rules)  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_firewall_rule_subgroups(async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param str name: name of resource
        :param bool verbose: True for verbose output
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: FirewallSubgroupListResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        return self.get_firewall_rule_subgroups_with_http_info(**kwargs)  # noqa: E501

    def get_firewall_rule_subgroups_with_http_info(self, **kwargs):  # noqa: E501
        """get_firewall_rule_subgroups  # noqa: E501

        Get a list of current firewall rules at subgroup (chained rules)  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_firewall_rule_subgroups_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param str name: name of resource
        :param bool verbose: True for verbose output
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: tuple(FirewallSubgroupListResponse, status_code(int), headers(HTTPHeaderDict))
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = ["name", "verbose"]  # noqa: E501
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        for key, val in six.iteritems(local_var_params["kwargs"]):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_firewall_rule_subgroups" % key
                )
            local_var_params[key] = val
        del local_var_params["kwargs"]

        collection_formats = {}

        path_params = {}

        query_params = []
        if "name" in local_var_params:
            query_params.append(("name", local_var_params["name"]))  # noqa: E501
        if "verbose" in local_var_params:
            query_params.append(("verbose", local_var_params["verbose"]))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # Authentication setting
        auth_settings = ["basicAuth"]  # noqa: E501

        return self.api_client.call_api(
            "/firewall/rules/subgroup",
            "GET",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type="FirewallSubgroupListResponse",  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get("async_req"),
            _return_http_data_only=local_var_params.get(
                "_return_http_data_only"
            ),  # noqa: E501
            _preload_content=local_var_params.get("_preload_content", True),
            _request_timeout=local_var_params.get("_request_timeout"),
            collection_formats=collection_formats,
        )

    def get_firewall_rules(self, **kwargs):  # noqa: E501
        """get_firewall_rules  # noqa: E501

        Get a list of current firewall rules  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_firewall_rules(async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: FirewallRuleListResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        return self.get_firewall_rules_with_http_info(**kwargs)  # noqa: E501

    def get_firewall_rules_with_http_info(self, **kwargs):  # noqa: E501
        """get_firewall_rules  # noqa: E501

        Get a list of current firewall rules  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_firewall_rules_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: tuple(FirewallRuleListResponse, status_code(int), headers(HTTPHeaderDict))
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = []  # noqa: E501
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        for key, val in six.iteritems(local_var_params["kwargs"]):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_firewall_rules" % key
                )
            local_var_params[key] = val
        del local_var_params["kwargs"]

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # Authentication setting
        auth_settings = ["basicAuth"]  # noqa: E501

        return self.api_client.call_api(
            "/firewall/rules",
            "GET",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type="FirewallRuleListResponse",  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get("async_req"),
            _return_http_data_only=local_var_params.get(
                "_return_http_data_only"
            ),  # noqa: E501
            _preload_content=local_var_params.get("_preload_content", True),
            _request_timeout=local_var_params.get("_request_timeout"),
            collection_formats=collection_formats,
        )

    def post_create_firewall_rule(
        self, create_firewall_rule_request, **kwargs
    ):  # noqa: E501
        """post_create_firewall_rule  # noqa: E501

        Adds a firewall rule to the VNS3 Controller's firewall  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.post_create_firewall_rule(create_firewall_rule_request, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param CreateFirewallRuleRequest create_firewall_rule_request: (required)
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: FirewallRuleOperationResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        return self.post_create_firewall_rule_with_http_info(
            create_firewall_rule_request, **kwargs
        )  # noqa: E501

    def post_create_firewall_rule_with_http_info(
        self, create_firewall_rule_request, **kwargs
    ):  # noqa: E501
        """post_create_firewall_rule  # noqa: E501

        Adds a firewall rule to the VNS3 Controller's firewall  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.post_create_firewall_rule_with_http_info(create_firewall_rule_request, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param CreateFirewallRuleRequest create_firewall_rule_request: (required)
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: tuple(FirewallRuleOperationResponse, status_code(int), headers(HTTPHeaderDict))
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = ["create_firewall_rule_request"]  # noqa: E501
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        for key, val in six.iteritems(local_var_params["kwargs"]):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method post_create_firewall_rule" % key
                )
            local_var_params[key] = val
        del local_var_params["kwargs"]
        # verify the required parameter 'create_firewall_rule_request' is set
        if (
            "create_firewall_rule_request" not in local_var_params
            or local_var_params["create_firewall_rule_request"] is None
        ):
            raise ApiValueError(
                "Missing the required parameter `create_firewall_rule_request` when calling `post_create_firewall_rule`"
            )  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if "create_firewall_rule_request" in local_var_params:
            body_params = local_var_params["create_firewall_rule_request"]
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # HTTP header `Content-Type`
        header_params[
            "Content-Type"
        ] = self.api_client.select_header_content_type(  # noqa: E501
            ["application/json"]
        )  # noqa: E501

        # Authentication setting
        auth_settings = ["basicAuth"]  # noqa: E501

        return self.api_client.call_api(
            "/firewall/rules",
            "POST",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type="FirewallRuleOperationResponse",  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get("async_req"),
            _return_http_data_only=local_var_params.get(
                "_return_http_data_only"
            ),  # noqa: E501
            _preload_content=local_var_params.get("_preload_content", True),
            _request_timeout=local_var_params.get("_request_timeout"),
            collection_formats=collection_formats,
        )

    def post_create_firewall_subgroup_rule(
        self, create_fw_subgroup_request, **kwargs
    ):  # noqa: E501
        """post_create_firewall_subgroup_rule  # noqa: E501

        Create a new firewall subgroup rules (rule chain)  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.post_create_firewall_subgroup_rule(create_fw_subgroup_request, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param CreateFWSubgroupRequest create_fw_subgroup_request: (required)
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: OneOfobjectobject
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        return self.post_create_firewall_subgroup_rule_with_http_info(
            create_fw_subgroup_request, **kwargs
        )  # noqa: E501

    def post_create_firewall_subgroup_rule_with_http_info(
        self, create_fw_subgroup_request, **kwargs
    ):  # noqa: E501
        """post_create_firewall_subgroup_rule  # noqa: E501

        Create a new firewall subgroup rules (rule chain)  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.post_create_firewall_subgroup_rule_with_http_info(create_fw_subgroup_request, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param CreateFWSubgroupRequest create_fw_subgroup_request: (required)
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: tuple(OneOfobjectobject, status_code(int), headers(HTTPHeaderDict))
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = ["create_fw_subgroup_request"]  # noqa: E501
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        for key, val in six.iteritems(local_var_params["kwargs"]):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method post_create_firewall_subgroup_rule" % key
                )
            local_var_params[key] = val
        del local_var_params["kwargs"]
        # verify the required parameter 'create_fw_subgroup_request' is set
        if (
            "create_fw_subgroup_request" not in local_var_params
            or local_var_params["create_fw_subgroup_request"] is None
        ):
            raise ApiValueError(
                "Missing the required parameter `create_fw_subgroup_request` when calling `post_create_firewall_subgroup_rule`"
            )  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if "create_fw_subgroup_request" in local_var_params:
            body_params = local_var_params["create_fw_subgroup_request"]
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # HTTP header `Content-Type`
        header_params[
            "Content-Type"
        ] = self.api_client.select_header_content_type(  # noqa: E501
            ["application/json"]
        )  # noqa: E501

        # Authentication setting
        auth_settings = ["basicAuth"]  # noqa: E501

        return self.api_client.call_api(
            "/firewall/rules/subgroup",
            "POST",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type="OneOfobjectobject",  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get("async_req"),
            _return_http_data_only=local_var_params.get(
                "_return_http_data_only"
            ),  # noqa: E501
            _preload_content=local_var_params.get("_preload_content", True),
            _request_timeout=local_var_params.get("_request_timeout"),
            collection_formats=collection_formats,
        )

    def put_reinitialize_fw_sets(self, reinit_firewall_request, **kwargs):  # noqa: E501
        """put_reinitialize_fw_sets  # noqa: E501

        Reinitialize all Firewall FWSet  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.put_reinitialize_fw_sets(reinit_firewall_request, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param ReinitFirewallRequest reinit_firewall_request: (required)
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        return self.put_reinitialize_fw_sets_with_http_info(
            reinit_firewall_request, **kwargs
        )  # noqa: E501

    def put_reinitialize_fw_sets_with_http_info(
        self, reinit_firewall_request, **kwargs
    ):  # noqa: E501
        """put_reinitialize_fw_sets  # noqa: E501

        Reinitialize all Firewall FWSet  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.put_reinitialize_fw_sets_with_http_info(reinit_firewall_request, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param ReinitFirewallRequest reinit_firewall_request: (required)
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = ["reinit_firewall_request"]  # noqa: E501
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        for key, val in six.iteritems(local_var_params["kwargs"]):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method put_reinitialize_fw_sets" % key
                )
            local_var_params[key] = val
        del local_var_params["kwargs"]
        # verify the required parameter 'reinit_firewall_request' is set
        if (
            "reinit_firewall_request" not in local_var_params
            or local_var_params["reinit_firewall_request"] is None
        ):
            raise ApiValueError(
                "Missing the required parameter `reinit_firewall_request` when calling `put_reinitialize_fw_sets`"
            )  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if "reinit_firewall_request" in local_var_params:
            body_params = local_var_params["reinit_firewall_request"]
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # HTTP header `Content-Type`
        header_params[
            "Content-Type"
        ] = self.api_client.select_header_content_type(  # noqa: E501
            ["application/json"]
        )  # noqa: E501

        # Authentication setting
        auth_settings = ["basicAuth"]  # noqa: E501

        return self.api_client.call_api(
            "/firewall/fwsets",
            "PUT",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get("async_req"),
            _return_http_data_only=local_var_params.get(
                "_return_http_data_only"
            ),  # noqa: E501
            _preload_content=local_var_params.get("_preload_content", True),
            _request_timeout=local_var_params.get("_request_timeout"),
            collection_formats=collection_formats,
        )

    def put_reinitialize_subgroups(self, reinit_request, **kwargs):  # noqa: E501
        """put_reinitialize_subgroups  # noqa: E501

        Reinitialize Firewall subgroups  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.put_reinitialize_subgroups(reinit_request, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param ReinitRequest reinit_request: (required)
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        return self.put_reinitialize_subgroups_with_http_info(
            reinit_request, **kwargs
        )  # noqa: E501

    def put_reinitialize_subgroups_with_http_info(
        self, reinit_request, **kwargs
    ):  # noqa: E501
        """put_reinitialize_subgroups  # noqa: E501

        Reinitialize Firewall subgroups  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.put_reinitialize_subgroups_with_http_info(reinit_request, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param ReinitRequest reinit_request: (required)
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = ["reinit_request"]  # noqa: E501
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        for key, val in six.iteritems(local_var_params["kwargs"]):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method put_reinitialize_subgroups" % key
                )
            local_var_params[key] = val
        del local_var_params["kwargs"]
        # verify the required parameter 'reinit_request' is set
        if (
            "reinit_request" not in local_var_params
            or local_var_params["reinit_request"] is None
        ):
            raise ApiValueError(
                "Missing the required parameter `reinit_request` when calling `put_reinitialize_subgroups`"
            )  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if "reinit_request" in local_var_params:
            body_params = local_var_params["reinit_request"]
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # HTTP header `Content-Type`
        header_params[
            "Content-Type"
        ] = self.api_client.select_header_content_type(  # noqa: E501
            ["application/json"]
        )  # noqa: E501

        # Authentication setting
        auth_settings = ["basicAuth"]  # noqa: E501

        return self.api_client.call_api(
            "/firewall/rules/subgroup",
            "PUT",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get("async_req"),
            _return_http_data_only=local_var_params.get(
                "_return_http_data_only"
            ),  # noqa: E501
            _preload_content=local_var_params.get("_preload_content", True),
            _request_timeout=local_var_params.get("_request_timeout"),
            collection_formats=collection_formats,
        )



# coding: utf-8

"""
    VNS3 Controller API

    Cohesive networks VNS3 API providing complete control of your network's addresses, routes, rules and edge  # noqa: E501

    The version of the OpenAPI document: 4.8
    Contact: solutions@cohesive.net
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from cohesivenet.configuration import Configuration


class WebhookIntegration(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "id": "int",
        "name": "str",
        "validate_cert": "bool",
        "created_at": "datetime",
        "updated_at": "datetime",
        "url": "str",
        "body": "str",
        "custom_properties": "list[WebhookCustomProperty]",
        "headers": "list[WebhookHeader]",
        "parameters": "list[WebhookParameter]",
        "events": "list[str]",
    }

    attribute_map = {
        "id": "id",
        "name": "name",
        "validate_cert": "validate_cert",
        "created_at": "created_at",
        "updated_at": "updated_at",
        "url": "url",
        "body": "body",
        "custom_properties": "custom_properties",
        "headers": "headers",
        "parameters": "parameters",
        "events": "events",
    }

    def __init__(
        self,
        id=None,
        name=None,
        validate_cert=None,
        created_at=None,
        updated_at=None,
        url=None,
        body=None,
        custom_properties=None,
        headers=None,
        parameters=None,
        events=None,
        local_vars_configuration=None,
    ):  # noqa: E501
        """WebhookIntegration - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._id = None
        self._name = None
        self._validate_cert = None
        self._created_at = None
        self._updated_at = None
        self._url = None
        self._body = None
        self._custom_properties = None
        self._headers = None
        self._parameters = None
        self._events = None
        self.discriminator = None

        if id is not None:
            self.id = id
        if name is not None:
            self.name = name
        if validate_cert is not None:
            self.validate_cert = validate_cert
        if created_at is not None:
            self.created_at = created_at
        if updated_at is not None:
            self.updated_at = updated_at
        if url is not None:
            self.url = url
        if body is not None:
            self.body = body
        if custom_properties is not None:
            self.custom_properties = custom_properties
        if headers is not None:
            self.headers = headers
        if parameters is not None:
            self.parameters = parameters
        if events is not None:
            self.events = events

    @property
    def id(self):
        """Gets the id of this WebhookIntegration.  # noqa: E501


        :return: The id of this WebhookIntegration.  # noqa: E501
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this WebhookIntegration.


        :param id: The id of this WebhookIntegration.  # noqa: E501
        :type: int
        """

        self._id = id

    @property
    def name(self):
        """Gets the name of this WebhookIntegration.  # noqa: E501


        :return: The name of this WebhookIntegration.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this WebhookIntegration.


        :param name: The name of this WebhookIntegration.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def validate_cert(self):
        """Gets the validate_cert of this WebhookIntegration.  # noqa: E501


        :return: The validate_cert of this WebhookIntegration.  # noqa: E501
        :rtype: bool
        """
        return self._validate_cert

    @validate_cert.setter
    def validate_cert(self, validate_cert):
        """Sets the validate_cert of this WebhookIntegration.


        :param validate_cert: The validate_cert of this WebhookIntegration.  # noqa: E501
        :type: bool
        """

        self._validate_cert = validate_cert

    @property
    def created_at(self):
        """Gets the created_at of this WebhookIntegration.  # noqa: E501


        :return: The created_at of this WebhookIntegration.  # noqa: E501
        :rtype: datetime
        """
        return self._created_at

    @created_at.setter
    def created_at(self, created_at):
        """Sets the created_at of this WebhookIntegration.


        :param created_at: The created_at of this WebhookIntegration.  # noqa: E501
        :type: datetime
        """

        self._created_at = created_at

    @property
    def updated_at(self):
        """Gets the updated_at of this WebhookIntegration.  # noqa: E501


        :return: The updated_at of this WebhookIntegration.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_at

    @updated_at.setter
    def updated_at(self, updated_at):
        """Sets the updated_at of this WebhookIntegration.


        :param updated_at: The updated_at of this WebhookIntegration.  # noqa: E501
        :type: datetime
        """

        self._updated_at = updated_at

    @property
    def url(self):
        """Gets the url of this WebhookIntegration.  # noqa: E501


        :return: The url of this WebhookIntegration.  # noqa: E501
        :rtype: str
        """
        return self._url

    @url.setter
    def url(self, url):
        """Sets the url of this WebhookIntegration.


        :param url: The url of this WebhookIntegration.  # noqa: E501
        :type: str
        """

        self._url = url

    @property
    def body(self):
        """Gets the body of this WebhookIntegration.  # noqa: E501


        :return: The body of this WebhookIntegration.  # noqa: E501
        :rtype: str
        """
        return self._body

    @body.setter
    def body(self, body):
        """Sets the body of this WebhookIntegration.


        :param body: The body of this WebhookIntegration.  # noqa: E501
        :type: str
        """

        self._body = body

    @property
    def custom_properties(self):
        """Gets the custom_properties of this WebhookIntegration.  # noqa: E501


        :return: The custom_properties of this WebhookIntegration.  # noqa: E501
        :rtype: list[WebhookCustomProperty]
        """
        return self._custom_properties

    @custom_properties.setter
    def custom_properties(self, custom_properties):
        """Sets the custom_properties of this WebhookIntegration.


        :param custom_properties: The custom_properties of this WebhookIntegration.  # noqa: E501
        :type: list[WebhookCustomProperty]
        """

        self._custom_properties = custom_properties

    @property
    def headers(self):
        """Gets the headers of this WebhookIntegration.  # noqa: E501


        :return: The headers of this WebhookIntegration.  # noqa: E501
        :rtype: list[WebhookHeader]
        """
        return self._headers

    @headers.setter
    def headers(self, headers):
        """Sets the headers of this WebhookIntegration.


        :param headers: The headers of this WebhookIntegration.  # noqa: E501
        :type: list[WebhookHeader]
        """

        self._headers = headers

    @property
    def parameters(self):
        """Gets the parameters of this WebhookIntegration.  # noqa: E501


        :return: The parameters of this WebhookIntegration.  # noqa: E501
        :rtype: list[WebhookHeader]
        """
        return self._parameters

    @parameters.setter
    def parameters(self, parameters):
        """Sets the parameters of this WebhookIntegration.


        :param parameters: The parameters of this WebhookIntegration.  # noqa: E501
        :type: list[WebhookHeader]
        """

        self._parameters = parameters

    @property
    def events(self):
        """Gets the events of this WebhookIntegration.  # noqa: E501


        :return: The events of this WebhookIntegration.  # noqa: E501
        :rtype: list[str]
        """
        return self._events

    @events.setter
    def events(self, events):
        """Sets the events of this WebhookIntegration.


        :param events: The events of this WebhookIntegration.  # noqa: E501
        :type: list[str]
        """

        self._events = events

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(
                    map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value)
                )
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict")
                        else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, WebhookIntegration):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, WebhookIntegration):
            return True

        return self.to_dict() != other.to_dict()
